@@ -1,5 +1,8 @@
 use std::error::Error;
 use std::fmt;
+use std::ops::Deref;
+use std::str::FromStr;
+use std::sync::OnceLock;
 
 use pyo3::exceptions::PyException;
 use pyo3::prelude::*;
@@ -176,7 +179,7 @@ macro_rules! py_schema_err {
 pub(crate) use py_schema_err;
 
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
-pub(crate) enum ExtraBehavior {
+pub enum ExtraBehavior {
     Allow,
     Forbid,
     Ignore,
@@ -197,12 +200,47 @@ impl ExtraBehavior {
         )?
         .flatten();
         let res = match extra_behavior.as_ref().map(|s| s.to_str()).transpose()? {
-            Some("allow") => Self::Allow,
-            Some("ignore") => Self::Ignore,
-            Some("forbid") => Self::Forbid,
-            Some(v) => return py_schema_err!("Invalid extra_behavior: `{}`", v),
+            Some(s) => Self::from_str(s)?,
             None => default,
         };
         Ok(res)
     }
 }
+
+impl FromStr for ExtraBehavior {
+    type Err = PyErr;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s {
+            "allow" => Ok(Self::Allow),
+            "forbid" => Ok(Self::Forbid),
+            "ignore" => Ok(Self::Ignore),
+            s => py_schema_err!("Invalid extra_behavior: `{}`", s),
+        }
+    }
+}
+
+/// A lazily-initialized value.
+///
+/// This is a basic replacement for `LazyLock` which is available only in Rust 1.80+.
+pub struct LazyLock<T> {
+    init: fn() -> T,
+    value: OnceLock<T>,
+}
+
+impl<T> Deref for LazyLock<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        self.value.get_or_init(self.init)
+    }
+}
+
+impl<T> LazyLock<T> {
+    pub const fn new(init: fn() -> T) -> Self {
+        Self {
+            init,
+            value: OnceLock::new(),
+        }
+    }
+}
