@@ -1,4 +1,5 @@
 use std::borrow::Cow;
+use std::sync::Arc;
 
 use pyo3::intern;
 use pyo3::prelude::*;
@@ -55,7 +56,7 @@ impl WhenUsed {
 
 #[derive(Debug)]
 pub struct FormatSerializer {
-    format_func: PyObject,
+    format_func: Py<PyAny>,
     formatting_string: Py<PyString>,
     when_used: WhenUsed,
 }
@@ -66,28 +67,28 @@ impl BuildSerializer for FormatSerializer {
     fn build(
         schema: &Bound<'_, PyDict>,
         config: Option<&Bound<'_, PyDict>>,
-        definitions: &mut DefinitionsBuilder<CombinedSerializer>,
-    ) -> PyResult<CombinedSerializer> {
+        definitions: &mut DefinitionsBuilder<Arc<CombinedSerializer>>,
+    ) -> PyResult<Arc<CombinedSerializer>> {
         let py = schema.py();
         let formatting_string: Bound<'_, PyString> = schema.get_as_req(intern!(py, "formatting_string"))?;
         if formatting_string.is_empty()? {
             ToStringSerializer::build(schema, config, definitions)
         } else {
-            Ok(Self {
+            Ok(CombinedSerializer::Format(Self {
                 format_func: py
                     .import(intern!(py, "builtins"))?
                     .getattr(intern!(py, "format"))?
                     .unbind(),
                 formatting_string: formatting_string.unbind(),
                 when_used: WhenUsed::new(schema, WhenUsed::JsonUnlessNone)?,
-            }
+            })
             .into())
         }
     }
 }
 
 impl FormatSerializer {
-    fn call(&self, value: &Bound<'_, PyAny>) -> Result<PyObject, String> {
+    fn call(&self, value: &Bound<'_, PyAny>) -> Result<Py<PyAny>, String> {
         let py = value.py();
         self.format_func
             .call1(py, (value, &self.formatting_string))
@@ -113,7 +114,7 @@ impl TypeSerializer for FormatSerializer {
         _include: Option<&Bound<'_, PyAny>>,
         _exclude: Option<&Bound<'_, PyAny>>,
         extra: &Extra,
-    ) -> PyResult<PyObject> {
+    ) -> PyResult<Py<PyAny>> {
         if self.when_used.should_use(value, extra) {
             self.call(value).map_err(PydanticSerializationError::new_err)
         } else {
@@ -171,11 +172,11 @@ impl BuildSerializer for ToStringSerializer {
     fn build(
         schema: &Bound<'_, PyDict>,
         _config: Option<&Bound<'_, PyDict>>,
-        _definitions: &mut DefinitionsBuilder<CombinedSerializer>,
-    ) -> PyResult<CombinedSerializer> {
-        Ok(Self {
+        _definitions: &mut DefinitionsBuilder<Arc<CombinedSerializer>>,
+    ) -> PyResult<Arc<CombinedSerializer>> {
+        Ok(CombinedSerializer::ToString(Self {
             when_used: WhenUsed::new(schema, WhenUsed::JsonUnlessNone)?,
-        }
+        })
         .into())
     }
 }
@@ -189,7 +190,7 @@ impl TypeSerializer for ToStringSerializer {
         _include: Option<&Bound<'_, PyAny>>,
         _exclude: Option<&Bound<'_, PyAny>>,
         extra: &Extra,
-    ) -> PyResult<PyObject> {
+    ) -> PyResult<Py<PyAny>> {
         if self.when_used.should_use(value, extra) {
             value.str().map(Into::into)
         } else {
