@@ -1,4 +1,5 @@
 use std::collections::VecDeque;
+use std::sync::Arc;
 
 use pyo3::intern;
 use pyo3::prelude::*;
@@ -15,7 +16,7 @@ use super::{build_validator, BuildValidator, CombinedValidator, DefinitionsBuild
 #[derive(Debug)]
 pub struct TupleValidator {
     strict: bool,
-    validators: Vec<CombinedValidator>,
+    validators: Vec<Arc<CombinedValidator>>,
     variadic_item_index: Option<usize>,
     min_length: Option<usize>,
     max_length: Option<usize>,
@@ -28,16 +29,16 @@ impl BuildValidator for TupleValidator {
     fn build(
         schema: &Bound<'_, PyDict>,
         config: Option<&Bound<'_, PyDict>>,
-        definitions: &mut DefinitionsBuilder<CombinedValidator>,
-    ) -> PyResult<CombinedValidator> {
+        definitions: &mut DefinitionsBuilder<Arc<CombinedValidator>>,
+    ) -> PyResult<Arc<CombinedValidator>> {
         let py = schema.py();
         let items: Bound<'_, PyList> = schema.get_as_req(intern!(py, "items_schema"))?;
-        let validators: Vec<CombinedValidator> = items
+        let validators: Vec<Arc<CombinedValidator>> = items
             .iter()
             .map(|item| build_validator(&item, config, definitions))
             .collect::<PyResult<_>>()?;
 
-        let mut validator_names = validators.iter().map(Validator::get_name).collect::<Vec<_>>();
+        let mut validator_names = validators.iter().map(|v| v.get_name()).collect::<Vec<_>>();
         let variadic_item_index: Option<usize> = schema.get_as(intern!(py, "variadic_item_index"))?;
         // FIXME add friendly schema error if item out of bounds
         if let Some(variadic_item_index) = variadic_item_index {
@@ -45,7 +46,7 @@ impl BuildValidator for TupleValidator {
         }
         let name = format!("tuple[{}]", validator_names.join(", "));
 
-        Ok(Self {
+        Ok(CombinedValidator::Tuple(Self {
             strict: is_strict(schema, config)?,
             validators,
             variadic_item_index,
@@ -53,7 +54,7 @@ impl BuildValidator for TupleValidator {
             max_length: schema.get_as(intern!(py, "max_length"))?,
             name,
             fail_fast: schema.get_as(intern!(py, "fail_fast"))?.unwrap_or(false),
-        }
+        })
         .into())
     }
 }
@@ -67,9 +68,9 @@ impl TupleValidator {
         py: Python<'py>,
         input: &(impl Input<'py> + ?Sized),
         state: &mut ValidationState<'_, 'py>,
-        output: &mut Vec<PyObject>,
+        output: &mut Vec<Py<PyAny>>,
         errors: &mut Vec<ValLineError>,
-        item_validators: &[CombinedValidator],
+        item_validators: &[Arc<CombinedValidator>],
         collection_iter: &mut NextCountingIterator<impl Iterator<Item = I>>,
         actual_length: Option<usize>,
         fail_fast: bool,
@@ -111,7 +112,7 @@ impl TupleValidator {
         errors: &mut Vec<ValLineError>,
         collection_iter: &mut NextCountingIterator<impl Iterator<Item = I>>,
         actual_length: Option<usize>,
-    ) -> ValResult<Vec<PyObject>> {
+    ) -> ValResult<Vec<Py<PyAny>>> {
         let expected_length = if self.variadic_item_index.is_some() {
             actual_length.unwrap_or(self.validators.len())
         } else {
@@ -244,8 +245,8 @@ impl TupleValidator {
     fn push_output_item<'py>(
         &self,
         input: &(impl Input<'py> + ?Sized),
-        output: &mut Vec<PyObject>,
-        item: PyObject,
+        output: &mut Vec<Py<PyAny>>,
+        item: Py<PyAny>,
         actual_length: Option<usize>,
     ) -> ValResult<()> {
         output.push(item);
@@ -272,7 +273,7 @@ impl Validator for TupleValidator {
         py: Python<'py>,
         input: &(impl Input<'py> + ?Sized),
         state: &mut ValidationState<'_, 'py>,
-    ) -> ValResult<PyObject> {
+    ) -> ValResult<Py<PyAny>> {
         // this validator does not yet support partial validation, disable it to avoid incorrect results
         state.allow_partial = false.into();
 
@@ -331,8 +332,8 @@ where
     T: BorrowInput<'py>,
     I: Input<'py> + ?Sized,
 {
-    type Output = ValResult<Vec<PyObject>>;
-    fn consume_iterator(self, mut iterator: impl Iterator<Item = PyResult<T>>) -> ValResult<Vec<PyObject>> {
+    type Output = ValResult<Vec<Py<PyAny>>>;
+    fn consume_iterator(self, mut iterator: impl Iterator<Item = PyResult<T>>) -> ValResult<Vec<Py<PyAny>>> {
         let mut iteration_error = None;
 
         let output = self.validator.validate_tuple_variable(
